#include <QtQuick/QQuickItem>
#include <QtQuick/QQuickView>
#include <QMainWindow>
#include <QLineEdit>
#include <QVBoxLayout>
#include <QGroupBox>
#include <QDebug>
#include <QTimer>
#include <QMenuBar>
#include <QMenu>
#include <QAction>
#include <QApplication>

#if QT_VERSION >= 0x050300
#  include <QQuickWidget>
#endif

class MainWindow : public QMainWindow
{
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);

public slots:
    void dumpFocus();
    void appStateChanged(Qt::ApplicationState s);
};

MainWindow::MainWindow(QWidget *parent)    : QMainWindow(parent)
{
    setWindowTitle(QT_VERSION_STR);
    setObjectName("MainWin");

    QMenu *fm = menuBar()->addMenu("File");
    fm->setObjectName("Menu");
    QAction *a = new QAction("Quit", this);
    a->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q));
    connect(a, SIGNAL(triggered()), qApp, SLOT(quit()));
    fm->addAction(a);

    QStringList args = QCoreApplication::arguments();

    QWidget *c = new QWidget;
    c->setObjectName("Central");
    QVBoxLayout *l = new QVBoxLayout(c);

    if (args.contains("-la")) {
        QLineEdit *le = new QLineEdit;
        le->setObjectName("LineEditAbove");
        l->addWidget(le);
    }

    const QUrl u = QUrl("qrc:/xyz.qml");

    if (!args.contains("-no-wc")) {
        QQuickView *qml_view = new QQuickView();
        qml_view->setObjectName("QQuickView");
        qml_view->setSource(u);
        QWidget *qml_container = QWidget::createWindowContainer(qml_view);
        qml_container->setObjectName("qml_container");
        qml_container->setMinimumSize(QSize(200,160));
        QGroupBox *gb = new QGroupBox("QWindowContainer");
        gb->setObjectName("WindowContainerGroupBox");
        QVBoxLayout *gbl = new QVBoxLayout(gb);
        gbl->addWidget(qml_container);
        l->addWidget(gb);
    }

#if QT_VERSION >= 0x050300
    if (args.contains("-qw")) {
        QGroupBox *gb = new QGroupBox("QQuickWidget");
        gb->setObjectName("QuickWidgetGroupBox");
        QVBoxLayout *gbl = new QVBoxLayout(gb);
        QQuickWidget *qw = new QQuickWidget(u, gb);
        gbl->addWidget(qw);
        l->addWidget(gb);
    }
#endif

    if (args.contains("-lb")) {
        QLineEdit *le = new QLineEdit;
        le->setObjectName("LineEditBelow");
        l->addWidget(le);
    }

    setCentralWidget(c);
    QTimer *t = new QTimer(this);
    t->setInterval(2000);
    connect(t, SIGNAL(timeout()), this, SLOT(dumpFocus()));
    connect(qApp, SIGNAL(applicationStateChanged(Qt::ApplicationState)), this, SLOT(appStateChanged(Qt::ApplicationState)));
    t->start();
}

void MainWindow::appStateChanged(Qt::ApplicationState s)
{
     qDebug() << __FUNCTION__ << s;
}

void MainWindow::dumpFocus()
{
    static int n =0;
    qDebug() << '#' << n++ <<"focusWindow" << QGuiApplication::focusWindow()
         << "focusWidget" << QApplication::focusWidget();
}

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    qDebug() << QT_VERSION_STR << QGuiApplication::platformName();
    MainWindow w;
    w.move(300, 300);
    w.show();
    return a.exec();
}

#include "main.moc"
